@@ -10,6 +10,7 @@ from tornado.httputil import parse_multipart_form_data
 from tornado.ioloop import IOLoop
 from tornado.gen import coroutine, Return, engine
 from tornado.concurrent import return_future, Future
+from toto.session import TotoSession
 from uuid import uuid4
 import logging
 
@@ -17,7 +18,7 @@ define("allow_origin", default="*", help="This is the value for the Access-Contr
 define("method_select", default="both", metavar="both|url|parameter", help="Selects whether methods can be specified via URL, parameter in the message body or both (default both)")
 define("bson_enabled", default=False, help="Allows requests to use BSON with content-type application/bson")
 define("msgpack_enabled", default=False, help="Allows requests to use MessagePack with content-type application/msgpack")
-define("hmac_enabled", default=True, help="Uses the x-toto-hmac header to verify authenticated requests.")
+define("hmac_enabled", default=False, help="Uses the x-toto-hmac header to verify authenticated requests.")
 
 class BatchHandlerProxy(object):
   '''A proxy to a handler, this class intercepts calls to ``handler.respond()`` in order to match the
@@ -62,18 +63,18 @@ class TotoHandler(RequestHandler):
   '''The handler is responsible for processing all requests to the server. An instance
   will be initialized for each incoming request and will handle authentication, session
   management and method delegation for you.
-  
+
   You can set the module to use for method delegation via the ``method_module`` parameter.
   Methods are modules that contain an invoke function::
 
     def invoke(handler, parameters)
-  
+
   The request handler will be passed as the first parameter to the invoke function and
   provides access to the server's database connection, the current session and other
   useful request properties. Request parameters will be passed as the second argument
   to the invoke function. Any return values from ``invoke()`` functions should be
   JSON serializable.
-  
+
   Toto methods are generally invoked via a POST request to the server with a JSON
   serialized object as the body. The body should contain two properties:
 
@@ -137,7 +138,7 @@ class TotoHandler(RequestHandler):
         else:
           raise TotoException(ERROR_MISSING_METHOD, "Missing method.")
       cls.__get_method_path = get_method_path
-    
+
     if options.use_cookies:
       import math
       set_cookie = options.secure_cookies and cls.set_secure_cookie or cls.set_cookie
@@ -147,14 +148,16 @@ class TotoHandler(RequestHandler):
         set_cookie(self, name='toto-session-id', value=self.session.session_id, expires_days=math.ceil(self.session.expires / (24.0 * 60.0 * 60.0)), domain=options.cookie_domain)
         return self.session
       cls.create_session = create_session
-      
+
       def retrieve_session(self, session_id=None):
         if not self.session or (session_id and self.session.session_id != session_id):
           headers = self.request.headers
           if not session_id:
             session_id = 'x-toto-session-id' in headers and headers['x-toto-session-id'] or get_cookie(self, 'toto-session-id')
           if session_id:
-            self.session = self.db_connection.retrieve_session(session_id, options.hmac_enabled and headers.get('x-toto-hmac'), options.hmac_enabled and 'x-toto-hmac' in headers and self.request.body or None)
+            self.session = self.db_connection.retrieve_session(session_id)
+          if options.hmac_enabled and self.session:
+            self._verify_hmac(self.session, self.request, headers)
         if self.session:
           set_cookie(self, name='toto-session-id', value=self.session.session_id, expires_days=math.ceil(self.session.expires / (24.0 * 60.0 * 60.0)), domain=options.cookie_domain)
         return self.session
@@ -169,7 +172,7 @@ class TotoHandler(RequestHandler):
       cls.error_info = error_info
     cls.__method_root = __import__(options.method_module)
     cls.__method_cache = {}
-      
+
   def __get_method_path(self, path, body):
     """The default method_select "both" (or any unsupported value) will
     call this method. The class method ``configure()`` will update this
@@ -235,7 +238,7 @@ class TotoHandler(RequestHandler):
     self.add_header('access-control-allow-origin', self.ACCESS_CONTROL_ALLOW_ORIGIN)
     self.add_header('access-control-allow-methods', ','.join(self.SUPPORTED_METHODS))
     self.add_header('access-control-expose-headers', 'x-toto-hmac')
-  
+
   @coroutine
   def head(self, path=None):
     self.headers_only = True
@@ -274,7 +277,7 @@ class TotoHandler(RequestHandler):
     else:
       yield self.process_request(path, self.body, self.body and 'parameters' in self.body and self.body['parameters'] or {})
     self._after_invoke(self.transaction_id)
-  
+
   @return_future
   @engine
   def batch_process_request(self, requests, callback):
@@ -294,7 +297,7 @@ class TotoHandler(RequestHandler):
         proxy.respond(result, error, allow_async=False)
 
   @return_future
-  @engine 
+  @engine
   def process_request(self, path, request_body, parameters, callback):
     self._request_callback = callback
     self.session = None
@@ -318,7 +321,7 @@ class TotoHandler(RequestHandler):
     * application/msgpack - requires msgpack-python
 
     The response will also contain any available session information.
-    
+
     To help with error handling in asynchronous methods, calling ``handler.respond(error=<your_error>)`` with a caught
     exception will trigger a normal Toto error response, log the error and finish the request. This is the same basic
     flow that is used internally when exceptions are raised from synchronous method calls.
@@ -349,8 +352,8 @@ class TotoHandler(RequestHandler):
       response_body = self.msgpack.dumps(response)
     else:
       response_body = json.dumps(response)
-    if self.session and options.hmac_enabled:
-      self.add_header('x-toto-hmac', base64.b64encode(hmac.new(str(self.session.user_id).lower(), response_body, hashlib.sha1).digest()))
+    if options.hmac_enabled and self.session:
+      self.add_header('x-toto-hmac', self._response_hmac(self.session, response_body))
     self.respond_raw(response_body, self.response_type)
 
   def respond_raw(self, body, content_type, finish=True):
@@ -408,21 +411,34 @@ class TotoHandler(RequestHandler):
     checking the password. This can be used to implement alternative authentication methods like Facebook, Twitter
     and Google+.
     '''
-    self.session = self.db_connection.create_session(user_id, password, verify_password)
+    self.session = self.db_connection.create_session(user_id, password, verify_password, key=options.hmac_enabled and TotoSession.generate_id())
     return self.session
 
+  def _verify_hmac(self, session, request, headers):
+    self.session.verify('x-toto-hmac' in headers and headers['x-toto-hmac'], request.method + request.uri + (request.body or ''))
+
+  def _response_hmac(self, session, response_body):
+    return self.session.hmac(session.session_id + response_body)
+
   def retrieve_session(self, session_id=None):
     '''Retrieve the session specified by the request headers (or if enabled, the request cookie) and store it
     in ``self.session``. Alternatively, pass a ``session_id`` to this function to retrieve that session explicitly.
+
+    If the ``hmac_enabled`` option is set to ``True``, Verify the session against the given request. Expects that
+    the request's ``x-toto-hmac`` header is the sha1 hmac of the request signature signed with the authenticated key.
+
+    The signature takes the form: ``method + uri + body or ''``
     '''
     if not self.session or (session_id and self.session.session_id != session_id):
       headers = self.request.headers
       if not session_id and 'x-toto-session-id' in headers:
-        session_id = 'x-toto-session-id' in headers and headers['x-toto-session-id'] or None
+        session_id = headers['x-toto-session-id']
       if session_id:
-        self.session = self.db_connection.retrieve_session(session_id, options.hmac_enabled and headers.get('x-toto-hmac'), options.hmac_enabled and 'x-toto-hmac' in headers and self.request.body or None)
+        self.session = self.db_connection.retrieve_session(session_id)
+      if options.hmac_enabled and self.session:
+        self._verify_hmac(self.session, self.request, headers)
     return self.session
-    
+
   def on_finish(self):
     while self.registered_event_handlers:
       self.deregister_event_handler(self.registered_event_handlers[0])
@@ -435,7 +451,7 @@ class TotoHandler(RequestHandler):
 
   @classmethod
   def set_before_handler(cls, handler):
-    '''Set the handler that will be called before any method invocation. This is useful for instrumentation. The handler will be 
+    '''Set the handler that will be called before any method invocation. This is useful for instrumentation. The handler will be
        called with ``handler, transaction_id, method``, where ``transaction_id`` is a UUID and ``method`` is the name of the
        invoked method. For batch requests each invocation will be tracked and logged separately, but there will also be a message
        logged with name ``"<batch>"`` which wraps the entire request. The default handler is a no op.
@@ -448,3 +464,17 @@ class TotoHandler(RequestHandler):
        where ``transaction_id`` is the UUID that was passed to the before handler. The default handler is a no op.
     '''
     cls._after_invoke = handler
+
+  @classmethod
+  def set_hmac_handler(cls, handler):
+    '''Set the hmac verification function. It will be called with ``handler(session, request, headers)`` and may be used to add
+    additional verification or to modify the signature used for validation, e.g. to confirm request time.
+    '''
+    cls._verify_hmac = handler
+
+  @classmethod
+  def set_response_hmac_handler(cls, handler):
+    '''Set the hmac generation function for responses. It will be called with ``handler(session, response_body)`` and may be used to add
+    to modify the signature used for validation. It must return a string.
+    '''
+    cls._response_hmac = handler
